#!/usr/bin/env python
"""
Python debugger prompt.
Enhanced version of Pdb, using a prompt-toolkit front-end.

Usage::

    from rp.prompt_toolkit.contrib.pdb import set_trace
    set_trace()
"""
from __future__ import unicode_literals, absolute_import, print_function
from pygments.lexers import PythonLexer
from pygments.token import Token

from rp.prompt_toolkit.buffer import Buffer, AcceptAction
from rp.prompt_toolkit.completion import Completer
from rp.prompt_toolkit.contrib.regular_languages.completion import GrammarCompleter
from rp.prompt_toolkit.contrib.regular_languages.validation import GrammarValidator
from rp.prompt_toolkit.document import Document
from rp.prompt_toolkit.enums import DEFAULT_BUFFER
from rp.prompt_toolkit.filters import IsDone, Condition
from rp.prompt_toolkit.interface import CommandLineInterface
from rp.prompt_toolkit.layout.containers import HSplit, Window, ConditionalContainer, FloatContainer, Float, VSplit, ScrollOffsets
from rp.prompt_toolkit.layout.controls import BufferControl, FillControl
from rp.prompt_toolkit.layout.dimension import LayoutDimension
from rp.prompt_toolkit.layout.lexers import Lexer, PygmentsLexer
from rp.prompt_toolkit.layout.margins import Margin, NumberredMargin, ScrollbarMargin
from rp.prompt_toolkit.layout.processors import ConditionalProcessor, HighlightSearchProcessor, HighlightSelectionProcessor
from rp.prompt_toolkit.layout.utils import split_lines
from rp.prompt_toolkit.shortcuts import create_eventloop
from rp.prompt_toolkit.validation import Validator

from rp.rp_ptpython.completer import PythonCompleter
from rp.rp_ptpython.layout import CompletionVisualisation
from rp.rp_ptpython.python_input import PythonInput
from rp.rp_ptpython.repl import embed
from rp.rp_ptpython.validator import PythonValidator

from .commands import commands_with_help, shortcuts
from .completers import PythonFileCompleter, PythonFunctionCompleter, BreakPointListCompleter, AliasCompleter, PdbCommandsCompleter
from .grammar import create_pdb_grammar
from .key_bindings import load_custom_pdb_key_bindings
from .layout import PdbPromptStyle, CallStack, format_stack_entry
from .toolbars import PdbShortcutsToolbar, SourceTitlebar, StackTitlebar, BreakPointInfoToolbar
from .completion_hints import CompletionHint
from .style import get_ui_style

import linecache
import os
import pdb
import six
import sys
import weakref


__all__ = (
    'PtPdb',
    'set_trace',
)


class DynamicCompleter(Completer):
    """
    Proxy to a real completer which we can change at runtime.
    """
    def __init__(self, get_completer_func):
        self.get_completer_func = get_completer_func

    def get_completions(self, document, complete_event):
        for c in self.get_completer_func().get_completions(document, complete_event):
            yield c


class DynamicValidator(Validator):
    """
    Proxy to a real validator which we can change at runtime.
    """
    def __init__(self, get_validator_func):
        self.get_validator_func = get_validator_func

    def validate(self, document):
        return self.get_validator_func().validate(document)


class PdbLexer(Lexer):
    def __init__(self):
        self.python_lexer = PygmentsLexer(PythonLexer)

    def lex_document(self, cli, document):
        parts = document.text.split(None, 1)
        first_word = parts[0] if parts else ''

        # When the first word is a PDB command:
        if first_word in shortcuts.keys() or first_word in commands_with_help.keys():
            # PDB:
            if cli.is_done:
                tokens = [
                    (Token.PdbCommand, ' %s ' % first_word),
                    (Token, ' '),
                    (Token, parts[1] if len(parts) > 1 else ''),
                ]
            else:
                tokens = [(Token.Text, document.text)]

            token_lines = list(split_lines(tokens))
            def get_line(lineno):
                return token_lines[lineno]
            return get_line

        # Otherwise, highlight as Python code.
        else:
            return self.python_lexer.lex_document(cli, document)


def get_line_prefix_tokens(is_break, is_current_line):
    """
    Return the tokens to show in the left margin of a source code listing.
    """
    if is_break:
        if is_current_line:
            return [
                (Token.Break, 'B'),
                (Token.CurrentLine, '->')
            ]
        else:
            return [(Token.Break, ' B ')]
    else:
        if is_current_line:
            return [
                (Token.CurrentLine, '->'),
                (Token, ' ')
            ]
        else:
            return [(Token, '   ')]


class SourceCodeMargin(Margin):
    """
    Margin that shows 'B' and '->' for breaks and the current line.
    """
    def __init__(self, ptpdb):
        self.ptpdb = ptpdb

    def get_width(self, cli, _=None):
        return 3

    def create_margin(self, cli, window_render_info, width, height):
        filename = self.ptpdb.curframe.f_code.co_filename
        breaklist = self.ptpdb.get_file_breaks(filename)
        curframe = self.ptpdb.curframe

        visible_line_to_input_line = window_render_info.visible_line_to_input_line

        result = []

        for y in range(window_render_info.window_height):
            lineno = visible_line_to_input_line.get(y)

            if lineno is not None:
                is_current_line = lineno + 1 == curframe.f_lineno
                is_break = (lineno + 1) in breaklist
                result.extend(get_line_prefix_tokens(is_break, is_current_line))

            result.append((Token, '\n'))

        return result

    def invalidation_hash(self, cli, document):
        filename = self.ptpdb.curframe.f_code.co_filename

        return (
            tuple(self.ptpdb.get_file_breaks(filename)),
            self.ptpdb.curframe.f_lineno
        )

import rp.r_iterm_comm as ric

class PtPdb(pdb.Pdb):
    def __init__(self):
        pdb.Pdb.__init__(self)

        # Cache for the grammar.
        self._grammar_cache = None  # (current_pdb_commands, grammar) tuple.

        self.completer = None
        self.validator = None
        self.lexer = None

        self._source_code_window = Window(
            BufferControl(
                buffer_name='source_code',
                lexer=PygmentsLexer(PythonLexer),
                input_processors=[
                    HighlightSearchProcessor(preview_search=True),
                    HighlightSelectionProcessor(),
                ],
            ),
            left_margins=[
                SourceCodeMargin(self),
                NumberredMargin(),
            ],
            right_margins=[ScrollbarMargin()],
            scroll_offsets=ScrollOffsets(top=max(2,ric.debug_height//2-3), bottom=max(2,ric.debug_height//2-3)),
            height=LayoutDimension(preferred=10))

        # Callstack window.
        callstack = CallStack(weakref.ref(self))
        self.callstack_focussed = False  # When True, show cursor there, and allow navigation through it.
        self.callstack_selected_frame = 0  # Top frame.

        show_pdb_content_filter = ~IsDone() & Condition(
                    lambda cli: not self.python_input.show_exit_confirmation)

        self.python_input = PythonInput(
            get_locals=lambda: self.curframe.f_locals,
            get_globals=lambda: self.curframe.f_globals,
            _completer=DynamicCompleter(lambda: self.completer),
            _validator=DynamicValidator(lambda: self.validator),
            _accept_action = self._create_accept_action(),
            _extra_buffers={'source_code': Buffer(read_only=True)},
            _input_buffer_height=LayoutDimension(min=2, max=4),
            _lexer=PdbLexer(),
            _extra_buffer_processors=[
                ConditionalProcessor(
                    processor=CompletionHint(),
                    filter=~IsDone())
                ],
            _extra_layout_body=ConditionalContainer(
                HSplit([
                    VSplit([
                        HSplit([
                            SourceTitlebar(weakref.ref(self)),
                            FloatContainer(
                                content=self._source_code_window,
                                floats=[
                                    Float(right=0, bottom=0,
                                          content=BreakPointInfoToolbar(weakref.ref(self)))
                                ]),
                        ]),
                        HSplit([
                            Window(width=LayoutDimension.exact(1),
                                   height=LayoutDimension.exact(1),
                                   content=FillControl('\u252c', token=Token.Toolbar.Title)),
                            Window(width=LayoutDimension.exact(1),
                                   content=FillControl('\u2502', token=Token.Separator)),
                        ]),
                        HSplit([
                            StackTitlebar(weakref.ref(self)),
                            Window(callstack,
                                   scroll_offsets=ScrollOffsets(top=max(2,ric.debug_height//2), bottom=max(2,ric.debug_height//2)),
                                   right_margins=[ScrollbarMargin()],
                                   height=LayoutDimension(preferred=ric.debug_height)),
                        ]),
                    ]),
                ]),
                filter=show_pdb_content_filter),
            _extra_toolbars=[
                ConditionalContainer(
                    PdbShortcutsToolbar(weakref.ref(self)),
                    show_pdb_content_filter)
            ],
            history_filename=os.path.expanduser('~/.ptpdb_history'),
        )

        # Override prompt style.
        self.python_input.all_prompt_styles['pdb'] = PdbPromptStyle(self._get_current_pdb_commands())
        self.python_input.prompt_style = 'pdb'

        # Override exit message.
        self.python_input.exit_message = 'Do you want to quit BDB? This raises BdbQuit.'

        # Set UI styles.
        self.python_input.ui_styles = {
            'ptpdb': get_ui_style(),
        }
        self.python_input.use_ui_colorscheme('ptpdb')

        # Set autocompletion style. (Multi-column works nicer.)
        self.python_input.completion_visualisation = CompletionVisualisation.MULTI_COLUMN

        # Load additional key bindings.
        load_custom_pdb_key_bindings(self, self.python_input.key_bindings_registry)

        self.cli = CommandLineInterface(
            eventloop=create_eventloop(),
            application=self.python_input.create_application())

    def _create_accept_action(self):
        """
        Create an AcceptAction for the input buffer that replaces shortcuts
        like 's' with the full command ('step') before returning it.
        """
        def handler(cli, buffer):
            # Get first part.
            parts = buffer.text.strip().split(None, 1)
            if len(parts) == 0:
                first, rest = '', ''
            elif len(parts) == 1:
                first, rest = parts[0], ''
            else:
                first, rest = parts

            # Replace text in buffer and return it.
            buffer.document = Document(shortcuts.get(first, first) + ' ' + rest)
            cli.set_return_value(buffer.document)
        return AcceptAction(handler)

    def cmdloop(self, intro=None):
        """
        Copy/Paste of pdb.Pdb.cmdloop. But using our own CommandLineInterface
        for reading input instead.
        """
        self.preloop()

        if intro is not None:
            self.intro = intro
        if self.intro:
            self.stdout.write(str(self.intro)+"\n")
        stop = None
        while not stop:
            if self.cmdqueue:
                line = self.cmdqueue.pop(0)
            else:
                if self.use_rawinput:
                    line = self._get_input()

            line = self.precmd(line)
            stop = self.onecmd(line)
            stop = self.postcmd(stop, line)
        self.postloop()

    def _get_current_pdb_commands(self):
        return (
            list(commands_with_help.keys()) +
            list(shortcuts.keys()) +
            list(self.aliases.keys()))

    def _create_grammar(self):
        """
        Return the compiled grammar for this PDB shell.

        The grammar of PDB depends on the available list of PDB commands (which
        depends on the currently defined aliases.) Therefor we generate a new
        grammar when it changes, but cache it otherwise. (It's still expensive
        to compile.)
        """
        pdb_commands = self._get_current_pdb_commands()

        if self._grammar_cache is None or self._grammar_cache[0] != pdb_commands:
            self._grammar_cache = [
                pdb_commands,
                create_pdb_grammar(pdb_commands)]

        return self._grammar_cache[1]

    def _get_input(self):
        """
        Read PDB input. Return input text.
        """
        # Reset multiline/paste mode every time.
        self.python_input.paste_mode = False
        self.python_input.currently_multiline = False

        # Set source code document.
        self._show_source_code(self.curframe.f_code.co_filename)

        self.cli.buffers[DEFAULT_BUFFER].document = Document('')

        # Select the current frame of the stack.
        for i, (frame, lineno) in enumerate(self.stack):
            if frame is self.curframe:
                self.callstack_selected_frame = i
                break

        # Set up a new completer and validator for the new grammar.
        g = self._create_grammar()

        self.completer = GrammarCompleter(g, completers={
            'enabled_breakpoint': BreakPointListCompleter(only_enabled=True),
            'disabled_breakpoint': BreakPointListCompleter(only_disabled=True),
            'alias_name': AliasCompleter(self),
            'python_code': PythonCompleter(lambda: self.curframe.f_globals, lambda: self.curframe.f_locals),
            'breakpoint': BreakPointListCompleter(),
            'pdb_command': PdbCommandsCompleter(self),
            'python_file': PythonFileCompleter(),
            'python_function': PythonFunctionCompleter(self),
        })
        self.validator = GrammarValidator(g, {
            'python_code': PythonValidator()
        })

        # Make sure not to start in Vi navigation mode.
        # self.python_input.key_bindings_manager.reset(self.cli)
        # self.cli.buffers[DEFAULT_BUFFER].reset()

        def pre_run():
            self._source_code_window.vertical_scroll = 100000 # source_code_doc.line_count

        try:
            return self.cli.run(reset_current_buffer=False, pre_run=pre_run).text
        except EOFError:
            # Turn Control-D key press into a 'quit' command.
            return 'quit'

    def _show_source_code(self, filename):
        """
        Show the source code in the `source_code` buffer.
        """
        source_code_doc = self._get_source_code_document(filename)
        self.cli.buffers['source_code']._set_text(source_code_doc.text + '\n')
        self.cli.buffers['source_code']._set_cursor_position(source_code_doc.cursor_position)

    def _get_source_code_document(self, filename):
        """
        Return source code around current line as string.
        """
        source_code = linecache.getlines(filename)
        if six.PY2:
            source_code = [l.decode('utf-8') for l in source_code]

        source_code = ''.join(source_code)
        document = Document(source_code)

        return Document(document.text, document.translate_row_col_to_index(
            row=self.curframe.f_lineno - 1, col=0))

    #
    # Methods overriden from Pdb, in order to add highlighting.
    #

    def postcmd(self, stop, line):
        """
        Override 'postcmd': (Insert whitespace.)
        """
        print('')
        return pdb.Pdb.postcmd(self, stop, line)

    def preloop(self):
        print('')
        return pdb.Pdb.preloop(self)

    def do_interact(self, args):
        """
        Interact: start interpreter.
        (Override the 'pdb' implementation. We call ptpython instead.)
        """
        print('')
        ns = self.curframe.f_globals.copy()
        ns.update(self.curframe_locals)
        embed(globals=ns)

    def error(self, msg):
        """
        Override default error handler from PDB.
        """
        self.cli.print_tokens([
            (Token.Pdb.Error, '  %s  \n' % msg)
        ])

    def print_stack_entry(self, frame_lineno, prompt_prefix=': '):
        """
        Override `print_stack_entry` of Pdb, in order to add highlighting.
        """
        frame, lineno = frame_lineno

        tokens = []
        tokens.extend(format_stack_entry(self, frame, lineno))
        tokens.append((Token, '\n'))

        self.cli.print_tokens(tokens)

    def do_list(self, arg):
        """
        Override `Pdb.do_list`: Add highlighting.
        """
        self.lastcmd = 'list'
        last = None
        if arg and arg != '.':
            try:
                if ',' in arg:
                    first, last = arg.split(',')
                    first = int(first.strip())
                    last = int(last.strip())
                    if last < first:
                        # assume it's a count
                        last = first + last
                else:
                    first = int(arg.strip())
                    first = max(1, first - 5)
            except ValueError:
                self.error('Error in argument: %r' % arg)
                return
        elif self.lineno is None or arg == '.':
            first = max(1, self.curframe.f_lineno - 5)
        else:
            first = self.lineno + 1
        if last is None:
            last = first + 10
        filename = self.curframe.f_code.co_filename
        breaklist = self.get_file_breaks(filename)
        try:
            lines = linecache.getlines(filename, self.curframe.f_globals)
            self._print_lines_2(lines, first, last, breaklist,
                              self.curframe)
            self.lineno = min(last, len(lines))
            if len(lines) < last:
                self.message('[EOF]')
        except KeyboardInterrupt:
            pass
    do_l = do_list

    def _print_lines_2(self, lines, start, end, breaks=(), frame=None):
        """
        Similar to `Pdb._print_lines`, except that this takes all the lines
        of the given file as input, it uses Pygments for the highlighting,
        it does slicing, and it prints everything in color.
        """
        if frame:
            current_lineno = frame.f_lineno
        else:
            current_lineno = exc_lineno = -1

        # Highlight everything. (Highlighting works much better from the
        # beginning of the file.)
        all_tokens = python_lexer.get_tokens(''.join(lines))

        # Slice lines.
        lines = list(split_lines(all_tokens))[start-1:end]

        # Add left margin. (Numbers + 'B' or '->'.)
        def add_margin(lineno, tokens):
            is_break = lineno in breaks
            is_current_line = lineno == current_lineno

            return get_line_prefix_tokens(is_break, is_current_line) \
                + [(Token.LineNumber, str(lineno).rjust(3) + ' ')] \
                + tokens + [(Token, '\n')]

        lines = [add_margin(i + start, tokens) for i, tokens in enumerate(lines)]

        for l in lines:
            self.cli.print_tokens(l)

    def message(self, msg):
        """ Print message to stdout. This function is present in Pdb for
        Python3, but not in Python2. """
        print(msg, file=self.stdout)


python_lexer = PythonLexer(
    stripnl=False,
    stripall=False,
    ensurenl=False)


def set_trace():
    PtPdb().set_trace(sys._getframe().f_back)

def set_trace_shallow(level=0):
    #Set trace, but without the upper frames...aka don't go into the pseudo_termianl internals while debugging...
    frame=sys._getframe().f_back
    for _ in range(level):
        frame=frame.f_back
    PtPdb().set_trace(frame)



"""
For internal use only.
"""
from __future__ import unicode_literals

from prompt_toolkit.mouse_events import MouseEventTypes
import re

__all__ = (
    'has_unclosed_brackets',
    'get_jedi_script_from_document',
    'document_is_multiline_python',
)


def has_unclosed_brackets(text):
    """
    Starting at the end of the string. If we find an opening bracket
    for which we didn't had a closing one yet, return True.
    """
    stack = []

    # Ignore braces inside strings
    text = re.sub(r'''('[^']*'|"[^"]*")''', '', text)  # XXX: handle escaped quotes.!

    for c in reversed(text):
        if c in '])}':
            stack.append(c)

        elif c in '[({':
            if stack:
                if ((c == '[' and stack[-1] == ']') or
                        (c == '{' and stack[-1] == '}') or
                        (c == '(' and stack[-1] == ')')):
                    stack.pop()
            else:
                # Opening bracket for which we didn't had a closing one.
                return True

    return False


def get_jedi_script_from_document(document, locals, globals):
    import jedi  # We keep this import in-line, to improve start-up time.
                 # Importing Jedi is 'slow'.

    try:
        return jedi.Interpreter(
            document.text,
            column=document.cursor_position_col,
            line=document.cursor_position_row + 1,
            path='input-text',
            namespaces=[locals, globals])
    except ValueError:
        # Invalid cursor position.
        # ValueError('`column` parameter is not in a valid range.')
        return None
    except AttributeError:
        # Workaround for #65: https://github.com/jonathanslenders/python-prompt-toolkit/issues/65
        # See also: https://github.com/davidhalter/jedi/issues/508
        return None
    except IndexError:
        # Workaround Jedi issue #514: for https://github.com/davidhalter/jedi/issues/514
        return None
    except KeyError:
        # Workaroud for a crash when the input is "u'", the start of a unicode string.
        return None
    except Exception:
        # Workaround for: https://github.com/jonathanslenders/ptpython/issues/91
        return None


_multiline_string_delims = re.compile('''[']{3}|["]{3}''')


def document_is_multiline_python(document):
    """
    Determine whether this is a multiline Python document.
    """
    def ends_in_multiline_string():
        """
        ``True`` if we're inside a multiline string at the end of the text.
        """
        delims = _multiline_string_delims.findall(document.text)
        opening = None
        for delim in delims:
            if opening is None:
                opening = delim
            elif delim == opening:
                opening = None
        return bool(opening)

    if '\n' in document.text or ends_in_multiline_string():
        return True

    def line_ends_with_colon():
        return document.current_line.rstrip()[-1:] == ':'

    # If we just typed a colon, or still have open brackets, always insert a real newline.
    if line_ends_with_colon() or \
            (document.is_cursor_at_the_end and
             has_unclosed_brackets(document.text_before_cursor)) or \
            document.text.startswith('@'):
        return True

    # If the character before the cursor is a backslash (line continuation
    # char), insert a new line.
    elif document.text_before_cursor[-1:] == '\\':
        return True

    return False


def if_mousedown(handler):
    """
    Decorator for mouse handlers.
    Only handle event when the user pressed mouse down.

    (When applied to a token list. Scroll events will bubble up and are handled
    by the Window.)
    """
    def handle_if_mouse_down(cli, mouse_event):
        if mouse_event.event_type == MouseEventTypes.MOUSE_DOWN:
            return handler(cli, mouse_event)
        else:
            return NotImplemented
    return handle_if_mouse_down



from __future__ import unicode_literals
from abc import ABCMeta, abstractmethod
from six import with_metaclass

__all__ = (
    'EventLoop',
    'INPUT_TIMEOUT',
)


#: When to trigger the `onInputTimeout` event.
INPUT_TIMEOUT = .5


class EventLoop(with_metaclass(ABCMeta, object)):
    """
    Eventloop interface.
    """
    def run(self, stdin, callbacks):
        """
        Run the eventloop until stop() is called. Report all
        input/timeout/terminal-resize events to the callbacks.

        :param stdin: :class:`~prompt_toolkit.input.Input` instance.
        :param callbacks: :class:`~prompt_toolkit.eventloop.callbacks.EventLoopCallbacks` instance.
        """
        raise NotImplementedError("This eventloop doesn't implement synchronous 'run()'.")

    def run_as_coroutine(self, stdin, callbacks):
        """
        Similar to `run`, but this is a coroutine. (For asyncio integration.)
        """
        raise NotImplementedError("This eventloop doesn't implement 'run_as_coroutine()'.")

    @abstractmethod
    def stop(self):
        """
        Stop the `run` call. (Normally called by
        :class:`~prompt_toolkit.interface.CommandLineInterface`, when a result
        is available, or Abort/Quit has been called.)
        """

    @abstractmethod
    def close(self):
        """
        Clean up of resources. Eventloop cannot be reused a second time after
        this call.
        """

    @abstractmethod
    def add_reader(self, fd, callback):
        """
        Start watching the file descriptor for read availability and then call
        the callback.
        """

    @abstractmethod
    def remove_reader(self, fd):
        """
        Stop watching the file descriptor for read availability.
        """

    @abstractmethod
    def run_in_executor(self, callback):
        """
        Run a long running function in a background thread. (This is
        recommended for code that could block the event loop.)
        Similar to Twisted's ``deferToThread``.
        """

    @abstractmethod
    def call_from_executor(self, callback, _max_postpone_until=None):
        """
        Call this function in the main event loop. Similar to Twisted's
        ``callFromThread``.

        :param _max_postpone_until: `None` or `time.time` value. For interal
            use. If the eventloop is saturated, consider this task to be low
            priority and postpone maximum until this timestamp. (For instance,
            repaint is done using low priority.)

            Note: In the past, this used to be a datetime.datetime instance,
                  but apparently, executing `time.time` is more efficient: it
                  does fewer system calls. (It doesn't read /etc/localtime.)
        """



import os
import sys
import time
from .config import *
from .scene import Scene
from .people import *
from .obstacles import *
from .scene_gen import *
from .input import *
from .gamefunctions import *
import rp
sys.path.append(rp.get_path_parent(__file__))

level = show_title_page()
# Initialize the scene
scene = Scene(SC_LENGTH, SC_WIDTH, SC_FULLWIDTH)
# Put mario at predefined initial position
mario = Mario(MARIO_LENGTH, MARIO_WIDTH)
mario.setPos(scene, MARIO_INIT_X, MARIO_INIT_Y)
# Generate scene : walls, pits, grass, clouds
generatescene(scene, level)
# Put enemies
putenemies(scene, level, Enemy1.enemies)
putcoins(scene, level, Coins.coins)

getinp = Get()
print(scene.displayScene(level))

while True:
    input = input_to(getinp, SPEED[level])
    # check if the player hasn't lost all lives
    check_lives(scene, Lives.lives, level)
    os.system('clear')
    generatescene(scene, level)
    # update all alive enemies
    update_enemies(scene, level, Enemy1.enemies)
    update_coins(scene, level, Coins.coins)
    # display the scene generated
    print(scene.displayScene(level))
    if input is not None:
        curhalf = scene.start + SCENE_MOVE_AFTER
        if input in ALLOWED_INPUTS:
            if input in ['d', 'C']:
                if(mario.y <= curhalf):
                    mario.move(input, scene)
                else:
                    scene.start += mario.step
                    mario.move(input, scene)
            else:
                mario.move(input, scene)

        if input == 'q':
            os.system('clear')
            sys.exit()
        else:
            mario.gravityfall(scene)
            chk = clashcheck(scene, mario, mario.x, mario.y)
            if chk == 0:
                pass
            elif chk == 2:
                killenemy(scene, mario.y, Enemy1.enemies)
            elif chk == 3:
                killenemy(scene, mario.y, Enemy1.enemies)
                Lives.lives -= 1
            elif chk == 4:
                collect_coin(scene, mario.y, Coins.coins)

    else:
        mario.gravityfall(scene)
        chk = clashcheck(scene, mario, mario.x, mario.y)
        if chk == 0:
            pass
        elif chk == 2:
            killenemy(scene, mario.y, Enemy1.enemies)
        elif chk == 3:
            killenemy(scene, mario.y, Enemy1.enemies)
            Lives.lives -= 1
        elif chk == 4:
            collect_coin(scene, mario.y, Coins.coins)

    if mario.y > scene.max_y:
        scene.max_y = mario.y
    scene.score = scene.max_y*SCORE_DIST + Enemy1.killed * \
        SCORE_KILLED + Coins.collected*SCORE_COINS



#TODO Fix that import ABC doesn't go to import Janky.ABC as ABC and instead goes to import Janky.ABC
#Will be used to systematically bundle other dependencies into RP so that it never ever breaks.....hopefully....
#TODO: We need a dependency graph....
def with_redirected_imports(code,prefix='rp.',module_names='prompt_toolkit'.split()):
    import re
    ans=code
    for _ in range(20):#Very overkill but its ok
        ans=re.sub(r'^(\s*import\s.*\,\s*)(%s)([^\w]|$)'%'|'.join(module_names),r'\1'+prefix+r'\2\3',ans,flags=re.MULTILINE)
    ans    =re.sub(r'^(\s*from\s+)(%s)([^\w]|$)'        %'|'.join(module_names),r'\1'+prefix+r'\2\3',ans,flags=re.MULTILINE)
    ans    =re.sub(r'^(\s*import\s+)(%s)([^\w]|$)'      %'|'.join(module_names),r'\1'+prefix+r'\2\3',ans,flags=re.MULTILINE)
    return ans
projectdir='Jenkins'
ans=make_directory(projectdir)
module_names=[]
def handle_module(module_name):
    import importlib as i
    m=i.import_module(module_name)
    module_names.append(module_name)
    module_folder=get_path_parent(m.__file__)
    if get_path_name(module_folder)!='site-packages':
        copy_directory(module_folder,projectdir)
    else:
        copy_file(m.__file__,projectdir)
def rewire():
    python_file_paths=get_file_paths(projectdir,recursive=True,file_extension_filter='py')
    for file_path in python_file_paths:  
        try:
            code=text_file_to_string(file_path)
            new_code=with_redirected_imports(code,projectdir+'.',module_names)
            if new_code!=code:
                fansi_print('CHANGED '+file_path,'blue')
            string_to_text_file(file_path,new_code)
            print('rewired '+file_path)
        except Exception as e:
            print_stack_trace(e)
    
rewire()
handle_module('six')
handle_module('pygments')
handle_module('prompt_toolkit')
handle_module('jupyter')
handle_module('IPython')
rewire()



#Copyright 2008-2009  Marius Muja (mariusm@cs.ubc.ca). All rights reserved.
#Copyright 2008-2009  David G. Lowe (lowe@cs.ubc.ca). All rights reserved.
#
#THE BSD LICENSE
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#1. Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
#IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
#THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.



from rp.libs.pyflann.exceptions import FLANNException
import numpy


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

def check(filename):
    with open(filename,"r") as f:
        line = f.readline().strip()
        if line[0]=='#':  # first line might be a comment
            line = f.readline().strip()
    
    values = line.split()    
    if len(values)==0:
        return False
    return all(map(is_number, values))
    

def save(dataset, filename):
    if not isinstance(dataset,numpy.ndarray):
        raise FLANNException("Can only save numpy arrays")    
    numpy.savetxt(filename,dataset, fmt="%g")

def load(filename, rows = -1, cols = -1, dtype = numpy.float32):
    dataset = numpy.loadtxt(filename, dtype=dtype)
    return dataset